#!/bin/bash
# Test suite of PLplot interactive stuff that cannot be tested with
# file output device drivers.

OVERALL_STATUS_CODE=0
INDEX_LIST="01 04 08 14 16 17 23"
DEVICE_LIST=
if [ "ON" = "ON" ] ; then
    DEVICE_LIST="$DEVICE_LIST xwin"
fi

if [ "ON" = "ON" ] ; then
    DEVICE_LIST="$DEVICE_LIST tk"
fi

if [ "ON" = "ON" ] ; then
    DEVICE_LIST="$DEVICE_LIST xcairo"
fi

if [ "ON" = "ON" ] ; then
    DEVICE_LIST="$DEVICE_LIST gcw"
fi

if [ "OFF" = "ON" ] ; then
    DEVICE_LIST="$DEVICE_LIST wxwidgets"
fi

for device in $DEVICE_LIST ; do
    for index in $INDEX_LIST ; do
	echo "c/x${index}c -dev $device"
	c/x${index}c -dev $device 2> test.error
	# Look for any status codes (segfaults, plexit) from the examples themselves
	status_code=$?
	if [ "$status_code" -ne 0 ] ; then
            echo "ERROR indicated by status code = $status_code for c/x${index}c -dev $device"
	    OVERALL_STATUS_CODE=$status_code
	fi
	cat test.error
	# Look for any PLPLOT ERROR messages from plwarn that do not result in an exit code.
	is_error=`grep -l 'PLPLOT ERROR' test.error`
	if [ -n "$is_error" ] ; then
            echo "ERROR indicated by 'PLPLOT ERROR' in stderr for c/x${index}c -dev $device" 
	    OVERALL_STATUS_CODE=1
	fi
    done
done

INDEX_LIST=
COUNT=0
if [ "OFF" = "ON" ] ; then
    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="c++"
    COMMAND[$COUNT]="./wxPLplotDemo"
    COUNT=$(( $COUNT + 1 ))
fi

if [ "ON" = "ON" ] ; then
    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="c"
    COMMAND[$COUNT]="./plplotcanvas_demo"
    COUNT=$(( $COUNT + 1 ))

    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="c"
    COMMAND[$COUNT]="./plplotcanvas_animation"
    COUNT=$(( $COUNT + 1 ))
fi

if [ "OFF" = "ON" ] ; then
    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="python"
    COMMAND[$COUNT]="./plplotcanvas_demo.py"
    COUNT=$(( $COUNT + 1 ))

    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="python"
    COMMAND[$COUNT]="./plplotcanvas_animation.py"
    COUNT=$(( $COUNT + 1 ))
fi

if [ "ON" = "ON" ] ; then
    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="tk"
    COMMAND[$COUNT]="./xtk01 -f tk01"
    COUNT=$(( $COUNT + 1 ))

    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="tk"
    COMMAND[$COUNT]="./xtk02 -f tk02"
    COUNT=$(( $COUNT + 1 ))

    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="tk"
    COMMAND[$COUNT]="plserver  -f tk03"
    COUNT=$(( $COUNT + 1 ))

    INDEX_LIST="$INDEX_LIST $COUNT"
    DIRECTORY[$COUNT]="tk"
    COMMAND[$COUNT]="./xtk04 -f tk04"
    COUNT=$(( $COUNT + 1 ))
fi

# execute all interactive commands set up by previous stanzas.
for index in $INDEX_LIST ; do
    pushd ${DIRECTORY[$index]}
    echo "${COMMAND[$index]}"
    ${COMMAND[$index]} 2> test.error
    # Look for any status codes (segfaults, plexit) from the examples themselves
    status_code=$?
    if [ "$status_code" -ne 0 ] ; then
        echo "ERROR indicated by status code = $status_code for ${COMMAND[$index]}"
	OVERALL_STATUS_CODE=$status_code
    fi
    cat test.error
    # Look for any PLPLOT ERROR messages from plwarn that do not result in an exit code.
    is_error=`grep -l 'PLPLOT ERROR' test.error`
    if [ -n "$is_error" ] ; then
        echo "ERROR indicated by 'PLPLOT ERROR' in stderr for ${COMMAND[$index]}" 
	OVERALL_STATUS_CODE=1
    fi
    popd
done

if [ "ON" = "ON" ] ; then
    cd tcl
    plserver <<EOF
plstdwin .
plxframe .plw
pack append . .plw {left expand fill}
source plgrid.tcl
proc 1 {} "plgrid .plw.plwin"
1
exit
EOF
    cd ../tk
    pwd
# Drop 2 to avoid 'invalid command name "plhlsrgb"' error.
# Drop 14 because multiple devices do not seem to work in this context.
# Drop 24 to avoid 'invalid command name "plschr"' error.
# Drop 31 since it produces empty plot (by design).
    plserver <<EOF
source tkdemos.tcl
1
3
4
5
6
7
8
9
10
11
12
13
15
16
17
18
19
20
21
22
23
25
26
27
28
29
30
exit
EOF
fi
if [ "$OVERALL_STATUS_CODE" -ne 0 ] ; then
    echo "A major error occurred for one of the interactive examples"
else
    echo "All interactive tests completed without major errors"
fi
exit $OVERALL_STATUS_CODE
